/*   
 *   $Id: txtask.c,v 1.14 2003/07/28 16:54:10 ondra Exp $
 * 
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001,2002,2003 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include "debtsRsc.h"
#include "float.h"
#include "db.h"
#include "currency.h"
#include "txtask.h"
#include "main.h"
#include "item.h"
#include "debts.h"

Boolean
txtask_dialog(Int16 titleid,Int16 nameid,char *text, Int16 size)
{
    FormPtr frm;
    FieldPtr fld;
    Int16 button;
    char title[MAX_INTL_LEN],name[MAX_INTL_LEN];

    SysCopyStringResource(title, titleid);
    SysCopyStringResource(name, nameid);

    frm = FrmInitForm(frmTxtAsk);
    FrmSetTitle(frm,title);
    FrmCopyLabel(frm,lblTxtAskName,name);
    fld = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fldTxtAsk));
    FldSetMaxChars(fld,size);
    if (StrLen(text) > 0) {
	FldInsert(fld,text,StrLen(text));
	FldSetSelection(fld,0,StrLen(text));
    }
    FrmSetFocus(frm,FrmGetObjectIndex(frm,fldTxtAsk));

    do {
	button = FrmDoDialog(frm);
	if (button == btnTxtAskCancel)
	    break;
	if (FldGetTextPtr(fld) && StrLen(FldGetTextPtr(fld)) > 0) {
	    StrCopy(text,FldGetTextPtr(fld));
	    break;
	}
    }while(1);

    FrmDeleteForm(frm);
    
    if (button == btnTxtAskCancel)
	return false;
    return true;
}

Boolean
txtask_dialog_cbox(Int16 titleid,Int16 nameid, Int16 descrid,
		   char *text, Int16 size,Boolean *indirect)
{
    FormPtr frm;
    FieldPtr fld;
    Int16 button;
    char title[MAX_INTL_LEN],name[MAX_INTL_LEN],descr[MAX_INTL_LEN];

    SysCopyStringResource(title, titleid);
    SysCopyStringResource(name, nameid);
    SysCopyStringResource(descr, descrid);

    frm = FrmInitForm(frmPrefsAsk);
    FrmSetTitle(frm,title);
    FrmCopyLabel(frm,lblTxtAskName,name);


    fld = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fldTxtAsk));
    FldSetMaxChars(fld,size);
    if (StrLen(text) > 0) {
	FldInsert(fld,text,StrLen(text));
	FldSetSelection(fld,0,StrLen(text));
    }
    FrmSetFocus(frm,FrmGetObjectIndex(frm,fldTxtAsk));

    CtlSetLabel(FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,chkPrefsIndirect)),
	       descr);
    CtlSetValue(FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,chkPrefsIndirect)),
		*indirect);

    do {
	button = FrmDoDialog(frm);
	if (button == btnTxtAskCancel)
	    break;
	if (FldGetTextPtr(fld) && StrLen(FldGetTextPtr(fld)) > 0) {
	    StrCopy(text,FldGetTextPtr(fld));
	    break;
	}
    }while(1);

    *indirect = CtlGetValue(FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,chkPrefsIndirect)));
    FrmDeleteForm(frm);
    
    if (button == btnTxtAskCancel)
	return false;

    return true;
}

Boolean
txtask_dialog_num(Int16 titleid,Int16 nameid,double *num)
{
    FormPtr frm;
    FieldPtr fld;
    Int16 button;
    char text[15];
    char title[MAX_INTL_LEN],name[MAX_INTL_LEN];

    SysCopyStringResource(title, titleid);
    SysCopyStringResource(name, nameid);

    frm = FrmInitForm(frmNumAsk);
    FrmSetTitle(frm,title);
    FrmCopyLabel(frm,lblNumAskName,name);
    fld = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fldNumAsk));

    float_to_str(text,*num,2);
    FldInsert(fld,text,StrLen(text));
    FldSetSelection(fld,0,StrLen(text));

    FrmSetFocus(frm,FrmGetObjectIndex(frm,fldNumAsk));

    do {
	button = FrmDoDialog(frm);
	if (button == btnNumAskCancel)
	    break;
	if (FldGetTextPtr(fld) && StrLen(FldGetTextPtr(fld)) > 0) {
	    *num = float_to_double(FldGetTextPtr(fld));
	    break;
	}
    }while(1);

    FrmDeleteForm(frm);
    
    if (button == btnNumAskCancel)
	return false;
    return true;
}


Boolean
txtask_dialog_rate(char *name,Int16 size,double *num)
{
    FormPtr frm;
    FieldPtr fldname,fldrate;
    Int16 button;
    char text[15];

    if (dbprefs.indirectcur) 
	frm = FrmInitForm(frmIndirectRateAsk);
    else
	frm = FrmInitForm(frmRateAsk);

    fldname = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fldRateAskName));
    fldrate = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fldRateAskRate));

    if (dbprefs.indirectcur) 
	StrPrintF(text,"1%s =",dbprefs.lcurrency);
    else
	StrCopy(text,dbprefs.lcurrency);
    FrmCopyLabel(frm,lblRate,text);

    if (dbprefs.indirectcur)
	float_to_str(text,1.0/(*num),4);
    else
	float_to_str(text,*num,4);
    FldInsert(fldrate,text,StrLen(text));
    FldInsert(fldname,name,StrLen(name));

    if (StrLen(name) == 0) 
	FrmSetFocus(frm,FrmGetObjectIndex(frm,fldRateAskName));
    else {
	FldSetSelection(fldrate,0,StrLen(text));
	FrmSetFocus(frm,FrmGetObjectIndex(frm,fldRateAskRate));
    }

    do {
	button = FrmDoDialog(frm);
	if (button == btnRateAskCancel)
	    break;
	if (FldGetTextPtr(fldname) && StrLen(FldGetTextPtr(fldname)) > 0 && \
	    FldGetTextPtr(fldrate) && StrLen(FldGetTextPtr(fldrate)) > 0) {
	    *num = float_to_double(FldGetTextPtr(fldrate));
	    if (dbprefs.indirectcur) 
		*num = 1.0/(*num);
	    StrCopy(name,FldGetTextPtr(fldname));
	    break;
	}
    }while(1);

    FrmDeleteForm(frm);
    
    if (button == btnRateAskCancel)
	return false;
    return true;
}

Boolean
txtask_dialog_person(char *name,Int16 size,double *num)
{
    FormPtr frm;
    FieldPtr fldname,fldrate;
    Int16 button;
    char text[15];

    frm = FrmInitForm(frmPersonAsk);
    fldname = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fldRateAskName));
    fldrate = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fldRateAskRate));

    float_to_str(text,*num,1);
    FldInsert(fldrate,text,StrLen(text));
    FldInsert(fldname,name,StrLen(name));

    if (StrLen(name) == 0) 
	FrmSetFocus(frm,FrmGetObjectIndex(frm,fldRateAskName));
    else {
	FldSetSelection(fldrate,0,StrLen(text));
	FrmSetFocus(frm,FrmGetObjectIndex(frm,fldRateAskRate));
    }

    do {
	button = FrmDoDialog(frm);
	if (button == btnRateAskCancel)
	    break;
	if (FldGetTextPtr(fldname) && StrLen(FldGetTextPtr(fldname)) > 0 && \
	    FldGetTextPtr(fldrate) && StrLen(FldGetTextPtr(fldrate)) > 0) {
	    *num = float_to_double(FldGetTextPtr(fldrate));
	    StrCopy(name,FldGetTextPtr(fldname));
	    break;
	}
    }while(1);

    FrmDeleteForm(frm);
    
    if (button == btnRateAskCancel)
	return false;
    return true;
}

static Boolean 
FilterFormHandleEvent (EventPtr e)
{
    Boolean handled = false;

    switch (e->eType) {
    case ctlSelectEvent:
	if (e->data.ctlSelect.controlID == popFilterDescription) {
	    item_choose_description(GetObjectPtr(lstFilterDescription),
				    GetObjectPtr(fldFilterDescription),
				    false);
	    handled = true;
	}
	break;
    default:
	break;
    }
    return handled;
}


Boolean
txtask_dialog_filter(filterType *gfilter)
{
    FormPtr frm;
    FieldPtr flddesc;
    ListPtr lstptr;
    ControlPtr popup;
    Int16 button;
    char **curmenu = NULL;
    Int16 i;

    frm = FrmInitForm(frmFilter);
    flddesc = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fldFilterDescription));
    lstptr = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,lstFilterCurrency));
    popup = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,popFilterCurrency));

    FldInsert(flddesc,gfilter->description,StrLen(gfilter->description));
    FrmSetFocus(frm,FrmGetObjectIndex(frm,fldFilterDescription));

    /* Set currency fields */
    curmenu = MemPtrNew(sizeof(char *) * (currencyList->nitems+1));
    curmenu[0] = "--";
    for (i=0;i < currencyList->nitems;i++)
	curmenu[i+1] = (char *)currencyList->list[i];

    
    LstSetListChoices(lstptr,curmenu,currencyList->nitems+1);
    LstSetHeight(lstptr,currencyList->nitems+1);
    if (gfilter->currency == -1) {
	LstSetSelection(lstptr,0);
	CtlSetLabel(popup,curmenu[0]);
    } else {
	for (i=0;i<currencyList->nitems;i++)
	    if (currencyList->list[i]->id == gfilter->currency) {
		LstSetSelection(lstptr,i+1);
		CtlSetLabel(popup,curmenu[i+1]);
		break;
	    }
    }

    FrmSetEventHandler(frm, FilterFormHandleEvent);
    button = FrmDoDialog(frm);
    if (button == btnFilterOK) {
	if (FldGetTextPtr(flddesc))
	    StrCopy(gfilter->description,FldGetTextPtr(flddesc));
	i = LstGetSelection(lstptr);
	if (i == 0)
	    gfilter->currency = -1;
	else 
	    gfilter->currency = currencyList->list[i-1]->id;
    }
    else if (button == btnFilterDisable) {
	StrCopy(gfilter->description,"");
	gfilter->currency = -1;
    }
    FrmDeleteForm(frm);
    MemPtrFree(curmenu);

    if (button == btnFilterCancel)
	return false;
    return true;
}

